# Copyright (c) 2024 - 2025 Noppanut Ploywong (@noppanut15) <noppanut.connect@gmail.com>
# Apache License 2.0 (see LICENSE file or http://www.apache.org/licenses/LICENSE-2.0)


"""
Version information for depthviz, will be replaced during the release process.
"""

# These version placeholders will be replaced later during substitution.
__version__ = "0.5.3b2"
__version_tuple__ = (0, 5, "3b2")
