# Copyright (c) 2024 - 2025 Noppanut Ploywong (@noppanut15) <noppanut.connect@gmail.com>
# Apache License 2.0 (see LICENSE file or http://www.apache.org/licenses/LICENSE-2.0)


"""
This module is the main entry point for the command line interface (CLI) of the DepthViz package.
"""

import sys
from depthviz.main import run

if __name__ == "__main__":
    sys.exit(run())
